<?php

use WHMCS\Database\Capsule;
use WHMCS\Module\Server\hosterapi\Exception\APIException;
use WHMCS\View\Menu\Item as MenuItem;

require __DIR__ . '/vendor/autoload.php';


function hosterapi_MetaData()
{
    return array(
        'DisplayName' => 'HosterAPI',
        'APIVersion' => 1.0,
        'RequiresServer' => true,
    );
}

function hosterapi_ClientAreaCustomButtonArray() {
    return [
        'Server starten' => 'start',
        'Server herunterfahren' => 'shutdown',
        'Server stoppen' => 'stop',
        'Server neustarten' => 'restart',
        'Server neuinstallieren' => 'reinstall',
        'Backup erstellen' => 'backupCreate',
        'Backup wiederherstellen' => 'backupRestore',
        'Passwort erneuern' => 'password',
        'rDNS Record setzen' => 'rdns',
    ];
}

function hosterapi_ConfigOptions()
{
    return [
        "mem" => [
            "FriendlyName" => "Arbeitsspeicher (RAM)",
            "Type" => "text",
            "Size" => "25",
            "Description" => "",
            "Default" => "1",
        ],
        "cpuCount" => [
            "FriendlyName" => "virtuelle Kerne (CPU)",
            "Type" => "text",
            "Size" => "25",
            "Description" => "",
            "Default" => "1",
        ],
        "disk" => [
            "FriendlyName" => "Speicherplatz (SSD)",
            "Type" => "text",
            "Size" => "25",
            "Description" => "",
            "Default" => "10",
        ],
        "ipCount" => [
            "FriendlyName" => "IPv4/v6",
            "Type" => "text",
            "Size" => "25",
            "Description" => "",
            "Default" => "1",
        ],
        "location" => [
            "FriendlyName" => "Standort",
            "Type" => "dropdown",
            "Options" => [
                "1" => "NTT (AMD EPYC)",
                "2" => "Skylink (Intel Xeon)",
                "3" => "NTT (AMD Ryzen)",
                "4" => "Skylink (AMD Ryzen)"
            ],
            "Description" => "Standort",
            "Default" => "1",
        ]
    ];
}

function hosterapi_ClientArea($params)
{
    if (hosterapi_serverexists($params)){
        $serverid = hosterapi_getserverid($params);
        $api = hosterapi_api($params);

        if ((isset($_GET['page']) && $_GET['page'] === "backups")) {
            return hosterapi_Backups($params);
        }

        if ((isset($_GET['page']) && $_GET['page'] === "reinstall")) {
            return hosterapi_ReinstallPage($params);
        }

        if ((isset($_GET['page']) && $_GET['page'] === "console")) {
            return hosterapi_VNCPage($params);
        }

        if ((isset($_GET['page']) && $_GET['page'] === "rdns")) {
            return hosterapi_rdnsPage($params);
        }

        $jobId = hosterapi_getJob($params);
        try {
            if ($jobId) {
                try {
                    $job = $api->job(intval($jobId));
                } catch (\Exception $e) {
                    $jobId = '';
                }
            }

            if ($jobId && $job->status !== 'finished') {
                $result = [
                    'job' => [
                        'failed' => $job->status === 'failed',
                        'progress' => $job->progress * 100
                    ]
                ];
            } else {
                if (!$jobId || $job->status === 'finished') hosterapi_setJob($params['serviceid'], '');

                try {
                    $pendingBackup = $api->serverBackupsPending($serverid);
                    if ($pendingBackup) {
                        $pbPercent = $pendingBackup->progress * 100;
                    }
                } catch (APIException $e) {}

                if ($pbPercent) {
                    $result = ['pb' => $pbPercent];
                } else {
                    $data = $api->serverStatus($serverid);
                    $config = $api->serverConfig($serverid);
                    $config->mem = $config->mem / 1024;
                    $config->password = hosterapi_getpassword($params);
                    $result = [
                        'cpu' => round($data->cpu * 100),
                        'mem' => round(($data->memfree / $data->mem) * 100),
                        'netout' => round(($data->netout / 107374182400) * 100),
                        //'uptime' => hosterapi_uptime($data->uptime),
                        'uptime' => $data->uptime,
                        'state' => $data->state,
                        'config' => (array)$config
                    ];
                }
            }
        } catch (\WHMCS\Module\Server\hosterapi\APIException $e) {
            $result = false;
        }

        return array(
            'templatefile' => 'clientarea',
            'vars' => array(
                'vars' => $params,
                'result' => $result ?: []
            ),
        );
    } else {
        return array(
            'templatefile' => 'clientarea',
            'vars' => array(
                'vars' => $params,
                'result' => null
            ),
        );
    }

}

function hosterapi_CreateAccount($params)
{   
    //Check if hosterapi_ServerID is already set - if YES then Return Error
    if (!hosterapi_serverexists($params)){
        //Get hosterapi API Client
        $api = hosterapi_api($params);
        //Template Default Debian 10.0
        $template = isset($params["configoptions"]["template"]) ?  $params["configoptions"]["template"] : "Debian10.0";
        // Get Core Options - If Not use configoption3
        $cpuCores = isset($params["configoptions"]["cpuCores"]) ? $params["configoptions"]["cpuCores"] : $params['configoption2'];
        // Get Memory Options - If Not use configoption1
        $mem = isset($params["configoptions"]["mem"]) ? $params["configoptions"]["mem"] : $params['configoption1'];
        // Get Disk Options - If Not use configoption2
        $disk = isset($params["configoptions"]["disk"]) ? $params["configoptions"]["disk"] : $params['configoption3'];
        // Get IP Options - If Not use configoption4
        $ipamount = isset($params["configoptions"]["ipCount"]) ? $params["configoptions"]["ipCount"] : $params['configoption4'];
        // Get Location - If Not use configoption5
        $location = isset($params["configoptions"]["location"]) ? $params["configoptions"]["location"] : $params['configoption5'];

        try {
            $result = $api->serverCreate(
                $template,
                $cpuCores,
                $mem,
                $disk,
                $ipamount,
				$location
            );
        } catch (APIException $e) {
            throw new Exception($e->getMessage());
        }

        if ($result->status === "finished") {
            hosterapi_activateserver($result->output->vmid, $params['serviceid'], $result->output->password);
            hosterapi_fillWHMCSIPs($result,$params['serviceid']);
            hosterapi_setJob($params['serviceid'], null);
            hosterapi_setPassword($params['serviceid'],$result->output->password);
            return "success";
        } else {
            return $result->output;
        }
    } else {
        $response = "Dieser Server existiert bereits";
        return $response; 
    }
   
}

function hosterapi_ChangePackage($params) {
    $serverid = hosterapi_getserverid($params);

    //Get hosterapi API Client
    $api = hosterapi_api($params);
    // Get Core Options - If Not use configoption3
    $cpuCores = isset($params["configoptions"]["cpuCores"]) ? $params["configoptions"]["cpuCores"] : $params['configoption2'];
    // Get Memory Options - If Not use configoption1
    $mem = isset($params["configoptions"]["mem"]) ? $params["configoptions"]["mem"] : $params['configoption1'];
    // Get Disk Options - If Not use configoption2
    $disk = isset($params["configoptions"]["disk"]) ? $params["configoptions"]["disk"] : $params['configoption3'];
    // Get IP Options - If Not use configoption4
    $ipamount = isset($params["configoptions"]["ipCount"]) ? $params["configoptions"]["ipCount"] : $params['configoption4'];

    try {
        $result = $api->serverChange(
            $serverid,
            $cpuCores,
            $mem,
            $disk,
            $ipamount
        );
    } catch (APIException $e) {
        return $e->getMessage();
    }

    if ($result) {
        hosterapi_setJob($params['serviceid'], null);
        try {
            $config = $api->serverConfig($serverid);
        } catch (APIException $e) {
            return $e->getMessage();
        }
        if ($config) {
            $ipObj = (object)[
                'output' => (object)[
                    'ip' => [],
                    'ipv6' => [],
                ]
            ];

            $ipObj->output->ip = array_splice($config->ips, 0, count($config->ips) / 2);
            $ipObj->output->ipv6 = $config->ips;

            hosterapi_fillWHMCSIPs($ipObj,$params['serviceid']);
        }
        return "success";
    } else {
        return $result->output;
    }
}

function hosterapi_Backups($params) {
    $serverid = hosterapi_getserverid($params);
    $api = hosterapi_api($params);

    $backups = (array)$api->serverBackups($serverid);

    try {
        $pendingBackup = $api->serverBackupsPending($serverid);
        if ($pendingBackup) {
            $pbPercent = $pendingBackup->progress * 100;
        } else {
            $pbPercent = false;
        }
    } catch (APIException $e) {
        $pbPercent = false;
    }

    return [
        'templatefile' => 'backups',
        'breadcrumb' => [
            'clientarea.php?action=productdetails&page=backups&id='.$params['serviceid'] => 'Backups',
        ],
        'vars' => [
            'backups' => $backups,
            'pb' => $pbPercent
        ],
    ];
}

function hosterapi_ReinstallPage($params) {
    $api = hosterapi_api($params);

    $templates = $api->serverTemplates();

    return [
        'templatefile' => 'reinstall',
        'breadcrumb' => [
            'clientarea.php?action=productdetails&page=reinstall&id='.$params['serviceid'] => 'Neuinstallation',
        ],
        'vars' => [
            'templates' => $templates,
        ],
    ];
}

function hosterapi_VNCPage($params) {
    $api = hosterapi_api($params);
    $serverid = hosterapi_getserverid($params);

    $novnc = $api->noVNC($serverid);

    return [
        'templatefile' => 'novnc',
        'breadcrumb' => [
            'clientarea.php?action=productdetails&page=console&id='.$params['serviceid'] => 'noVNC Konsole',
        ],
        'vars' => [
            'novnc' => $novnc,
        ],
    ];
}

function hosterapi_rdnsPage($params) {
    $api = hosterapi_api($params);
    $serverid = hosterapi_getserverid($params);
    $pattern = '/\b(?:\d{1,3}\.){3}\d{1,3}\b/';
    $ips = $api->serverConfig($serverid)->ips;
    $ipv4 = [];
    foreach ($ips as $ip) {
        preg_match_all($pattern, $ip, $matchedIp);
        $ipv4 = array_merge($ipv4, $matchedIp[0]);
    }
    return [
        'templatefile' => 'rdns',
        'breadcrumb' => [
            'clientarea.php?action=productdetails&page=rdns&id='.$params['serviceid'] => 'rDNS Verwaltung',
        ],
        'vars' => [
            'ips' => $ipv4,
        ],
    ];
}


function hosterapi_TerminateAccount($params)
{
    if (hosterapi_serverexists($params)){
        $serverid = hosterapi_getserverid($params);

        $api = hosterapi_api($params);

        $api->serverDelete(intval($serverid), true);

        //Clear all Data.
        hosterapi_clearData($params['serviceid']);

        return "success";
    } else {
        return "There is no Server!";
    }
}

add_hook('ClientAreaPrimarySidebar', 1, function (MenuItem $primarySidebar) {
    $id = $_GET["id"];
    $detailsMenu = $primarySidebar->getChild("Service Details Overview");
    $actionsMenu = $primarySidebar->getChild("Service Details Actions");

    if (!is_null($actionsMenu)) {
        $actionsMenu->setChildren(array_filter($actionsMenu->getChildren(), function ($key){
            return strpos($key, 'Custom Module Button') === false;
        }, ARRAY_FILTER_USE_KEY));
    }

    if (!is_null($detailsMenu)) {
        $detailsMenu->removeChild('Information');

        $detailsMenu->addChild('hosterapi-overview', [
                'name'       => 'hosterapi-overview',
                'label'      => "Informationen",
                'uri'        => "clientarea.php?action=productdetails&id=$id",
                'order'      => 10,
                'icon'       => 'fa-eye',
                'attributes' => []
            ]
        );

        $detailsMenu->addChild('hosterapi-backups', [
                'name'       => 'hosterapi-backups',
                'label'      => "Backups",
                'uri'        => "clientarea.php?action=productdetails&page=backups&id=$id",
                'order'      => 20,
                'icon'       => 'fa-save',
                'attributes' => []
            ]
        );

        $detailsMenu->addChild('hosterapi-reinstall', [
                'name'       => 'hosterapi-reinstall',
                'label'      => "Neuinstallation",
                'uri'        => "clientarea.php?action=productdetails&page=reinstall&id=$id",
                'order'      => 30,
                'icon'       => 'fa-sync-alt',
                'attributes' => []
            ]
        );

        $detailsMenu->addChild('hosterapi-console', [
                'name'       => 'hosterapi-console',
                'label'      => "noVNC Konsole",
                'uri'        => "clientarea.php?action=productdetails&page=console&id=$id",
                'order'      => 40,
                'icon'       => 'fa-desktop-alt',
                'attributes' => []
            ]
        );

        $detailsMenu->addChild('hosterapi-rdns', [
                'name'       => 'hosterapi-rdns',
                'label'      => "rDNS Verwaltung",
                'uri'        => "clientarea.php?action=productdetails&page=rdns&id=$id",
                'order'      => 10,
                'icon'       => 'fa-address-book',
                'attributes' => []
            ]
        );

        if (isset($_GET['page'])) {
            if ($_GET['page'] === 'backups') {
                $detailsMenu->getChild('hosterapi-backups')->setClass('active');
            } else if ($_GET['page'] === 'reinstall') {
                $detailsMenu->getChild('hosterapi-reinstall')->setClass('active');
            } else if ($_GET['page'] === 'console') {
                $detailsMenu->getChild('hosterapi-console')->setClass('active');
            } else if ($_GET['page'] === 'rdns') {
                $detailsMenu->getChild('hosterapi-rdns')->setClass('active');
            } else {
                $detailsMenu->getChild('hosterapi-overview')->setClass('active');
            }
        } else {
            $detailsMenu->getChild('hosterapi-overview')->setClass('active');
        }
    }
});

add_hook("ClientAreaPage", 1, function($vars){

    switch($_GET['page']) {
        case 'reinstall':
            $vars['breadcrumb'][] = [
                'link' => 'clientarea.php?action=productdetails&page=reinstall&id='.$_GET['id'],
                'page' => 'Neuinstallation'
            ];
            break;
        case 'backups':
            $vars['breadcrumb'][] = [
                'link' => 'clientarea.php?action=productdetails&page=backups&id='.$_GET['id'],
                'page' => 'Backups'
            ];
            break;
        case 'console':
            $vars['breadcrumb'][] = [
                'link' => 'clientarea.php?action=productdetails&page=console&id='.$_GET['id'],
                'page' => 'noVNC Konsole'
            ];
            break;
        case 'rdns':
            $vars['breadcrumb'][] = [
                'link' => 'clientarea.php?action=productdetails&page=rdns&id='.$_GET['id'],
                'page' => 'rDNS'
            ];
            break;
    }

    # Return Updated Breadcrumb To Smarty
    return array("breadcrumb" => $vars['breadcrumb']);

});


// Server functions


function hosterapi_stop($params)
{
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $api->serverStop(intval($serverid));

    return "success";
}

function hosterapi_start($params)
{
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $api->serverStart(intval($serverid));

    return "success";
}

function hosterapi_shutdown($params)
{
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $api->serverShutdown(intval($serverid));

    return "success";
}

function hosterapi_restart($params)
{
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $api->serverRestart(intval($serverid));

    return "success";
}

function hosterapi_backupCreate($params)
{
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $api->serverBackupsCreate(intval($serverid));

    sleep(3);

    return "success";
}

function hosterapi_backupRestore($params)
{
    if (!isset($_POST['backup'])) {
        return "no backup defined";
    }

    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $api->serverBackupsRestore(intval($serverid), $_POST['backup']);

    sleep(3);

    return "success";
}

function hosterapi_reinstall($params) {
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $jobId = hosterapi_getJob($params);

    if ($jobId) {
        return "job running";
    }

    $task = $api->serverReinstall(intval($serverid), empty($_POST['template']) ? null : $_POST['template']);

    hosterapi_setJob($params['serviceid'], $task->jobId);

    header('Location: clientarea.php?action=productdetails&id='.$params['serviceid']);
    exit;
}

function hosterapi_password($params) {
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $pw = $api->serverPasswordReset(intval($serverid));

    if ($pw) {
        hosterapi_setPassword($params['serviceid'], $pw->password);
    }

    header('Location: clientarea.php?action=productdetails&id='.$params['serviceid']);
    exit;
}

function hosterapi_rdns($params) {
    $serverid = hosterapi_getserverid($params);

    $api = hosterapi_api($params);

    $result = $api->setRdns(intval($serverid), $_POST['ip'], $_POST['hostname']);

//    header('Location: clientarea.php?action=productdetails&id='.$params['serviceid']);
//    exit;
}

// Utilities

function hosterapi_api($params)
{
    return new WHMCS\Module\Server\hosterapi\Client\Client(trim($params['serveraccesshash']), !in_array($params['serverhostname'], ['prod', 'production', 'live']));
}

function hosterapi_getserverid($params)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);

    $statement = $pdo->prepare("SELECT hosterapi_serverid FROM tblhosting WHERE id = ?");
    $statement->execute(array($params['serviceid']));
    return $statement->fetch(PDO::FETCH_ASSOC)['hosterapi_serverid'];
}

function hosterapi_getpassword($params)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);

    $statement = $pdo->prepare("SELECT hosterapi_serverpassword FROM tblhosting WHERE id = ?");
    $statement->execute(array($params['serviceid']));
    return $statement->fetch(PDO::FETCH_ASSOC)['hosterapi_serverpassword'];
}

function hosterapi_activateserver($serverid, $queryid, $password)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);

    $statement = $pdo->prepare("UPDATE tblhosting SET hosterapi_serverid = ?, `domainstatus` = ? WHERE id = ?");
    hosterapi_setPassword($queryid,$password);
    $statement->execute(array($serverid, 'Active', $queryid));
    return true;
}

function hosterapi_getJob($params)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);

    $statement = $pdo->prepare("SELECT `hosterapi_task` FROM tblhosting WHERE id = ?");
    $statement->execute(array($params['serviceid']));
    return $statement->fetch(PDO::FETCH_ASSOC)['hosterapi_task'];
}

function hosterapi_setJob($queryid, $job)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);

    $statement = $pdo->prepare("UPDATE tblhosting SET `hosterapi_task` = ? WHERE id = ?");
    $statement->execute(array($job, $queryid));

    return true;
}

function hosterapi_setPassword($queryid, $pw)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);
    $command = 'EncryptPassword';
    $postData = [
        'password2' => $pw
    ];
    $results = localAPI($command, $postData);
    $encryptedwhmcs = $results["password"];
    $statement = $pdo->prepare("UPDATE tblhosting SET `hosterapi_serverpassword` = ? , `password` = ? WHERE id = ?");
    $statement->execute(array($pw, $encryptedwhmcs, $queryid));

    return true;
}

function hosterapi_checkcolumn($pdo)
{
    try {
        $statement = $pdo->prepare("ALTER TABLE `tblhosting` ADD (`hosterapi_serverid` INT(25) NOT NULL, `hosterapi_serverpassword` VARCHAR(255) NULL, `hosterapi_task` VARCHAR(255) NULL)");
        $statement->execute(array());
    } catch (PDOException $e) {
    }
}

function hosterapi_uptime($str)
{
    $num   = floatval($str);
    $secs  = fmod($num, 60); $num = intdiv($num, 60);
    $mins  = $num % 60;      $num = intdiv($num, 60);
    $hours = $num;

    if ($secs < 10) $secs = "0$secs";
    if ($mins < 10) $mins = "0$mins";
    if ($hours < 10) $hours = "0$hours";

    return "$hours:$mins:$secs";
}

function hosterapi_serverexists($params)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);

    $statement = $pdo->prepare("SELECT * FROM tblhosting WHERE id = ?");
    $statement->execute(array($params['serviceid']));
    $serverid = $statement->fetch(PDO::FETCH_ASSOC)['hosterapi_serverid'];
    if ($serverid != null && $serverid != 0){
        return true;
    } else {
        return false;
    }
}

function hosterapi_fillWHMCSIPs($apiresult,$whmcsid)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);
    if (count($apiresult->output->ips) > 1){
        $assignedips = implode(",",$apiresult->output->ips);
        $assignedips = $assignedips . "," . implode(",",$apiresult->output->ipsv6);
    } else {
        $assignedips = $apiresult->output->ipsv6[0];
    }
    $dedicatedip = $apiresult->output->ips[0];
    $statement = $pdo->prepare("UPDATE tblhosting SET dedicatedip = ?, assignedips = ? WHERE id = ?");
    $statement->execute(array($dedicatedip,$assignedips,$whmcsid));
    return true;
}
function hosterapi_clearWHMCSIPs($apiresult,$whmcsid)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);
    $statement = $pdo->prepare("UPDATE tblhosting SET dedicatedip = ?, assignedips = ? WHERE id = ?");
    $statement->execute(array("","",$whmcsid));
    return true;
}
function hosterapi_clearData($whmcsid)
{
    $pdo = Capsule::connection()->getPdo();
    hosterapi_checkcolumn($pdo);
    $statement = $pdo->prepare("UPDATE tblhosting SET dedicatedip = ?, assignedips = ? , hosterapi_serverid = ? , hosterapi_serverpassword = ? , password = ? , hosterapi_task = ? WHERE id = ?");
    $statement->execute(array("","","0","","",null,$whmcsid));
    return true;
}

