<?php

namespace WHMCS\Module\Server\hosterapi;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use WHMCS\Module\Server\hosterapi\Exception\APIException;

class API {

    private $apiToken;
    protected $client;
    protected $apiVersion = 'v1';

    public function __construct(string $apiToken, bool $sandbox = false)
    {
        global $CONFIG;
        $this->apiToken = $apiToken;
        if (strpos($CONFIG['Version'], '8') !== false) {
            $this->client = new Client([
                'base_uri' => $sandbox ? "https://reseller-sandbox.hosterapi.de/api/$this->apiVersion/" : "https://reseller.hosterapi.de/api/$this->apiVersion/",
                'headers' => [
                    'Accept' => 'application/json',
                    'Content-Type' => 'application/json',
                    'Authorization' => 'Bearer ' . $this->apiToken,
                    'User-Agent' => 'HosterAPI-WHMCS/1.0 guzzle'
                ]
            ]);
        } else {
            $this->client = new Client([
                'base_url' => $sandbox ? "https://reseller-sandbox.hosterapi.de/api/$this->apiVersion/" : "https://reseller.hosterapi.de/api/$this->apiVersion/",
                'defaults' => [
                    'headers' => [
                        'Accept' => 'application/json',
                        'Content-Type' => 'application/json',
                        'Authorization' => 'Bearer ' . $this->apiToken,
                        'User-Agent' => 'HosterAPI-WHMCS/1.0 guzzle'
                    ]
                ]
            ]);
        }
    }

    public function get(string $request, array $data = [])
    {
        return $this->request('GET', $request, $data);
    }

    public function post(string $request, array $data, array $bodyData)
    {
        return $this->request('POST', $request, $data, $bodyData);
    }

    public function put(string $request, array $data = [], array $bodyData = [])
    {
        return $this->request('PUT', $request, $data, $bodyData);
    }

    public function delete(string $request, array $data = [])
    {
        return $this->request('DELETE', $request, $data);
    }

    protected function request(string $method, string $request, array $queryData, array $bodyData = []) {
        global $CONFIG;
        if (strpos($CONFIG['Version'], '8') !== false) {
            return $this->requestV8($method, $request, $queryData, $bodyData);
        }

        $query = [
            'query' => $queryData
        ];

        if ($method !== 'GET') {
            $query['json'] = $bodyData;
        }

        try {
            $rq = $this->client->createRequest($method, $request, $query);
            $rp = $this->client->send($rq);
        } catch (\Exception $e) {
            throw new APIException($e->getMessage(), 0 ,$e);
        }

        $resp = json_decode((string)$rp->getBody());

        if ($rp->getStatusCode() > 400 && $resp->error !== false) {
            throw new APIException(json_decode((string)$rp->getBody())->error);
        }

        return $resp->result;
    }

    protected function requestV8(string $method, string $request, array $queryData, array $bodyData = []) {
        $query = [
            'query' => $queryData
        ];

        if ($method !== 'GET') {
            $query['json'] = $bodyData;
        }

        try {
            $rq = $this->client->request($method, $request, $query);
        } catch (\Exception $e) {
            throw new APIException($e->getMessage(), 0 ,$e);
        }

        $resp = json_decode((string)$rq->getBody());

        if ($rq->getStatusCode() > 400 && $resp->error !== false) {
            throw new APIException(json_decode((string)$rq->getBody())->error);
        }

        return $resp->result;
    }
}
