<?php

namespace WHMCS\Module\Server\hosterapi\Client;

use WHMCS\Module\Server\hosterapi\API;
use WHMCS\Module\Server\hosterapi\Exception\APIException;

class Client extends API {

    public function serverCreate(
        string $template,
        int $cpuCores,
        int $mem,
        string $disk,
        int $ipCount,
        string $location
    ) {
        $job = $this->post('datacenter/server', [], [
            "template" => $template,
            "cpuCores" => $cpuCores,
            "mem" => $mem * 1024,
            "disk" => $disk . "G",
            "ipCount" => $ipCount,
            "location" => $location
        ]);
        $jobStatus = $this->get("job/$job->jobId");
        while ($jobStatus && !in_array($jobStatus->status, ['finished', 'failed'])) {
            $jobStatus = $this->get("job/$job->jobId");
            sleep(1);
        }

        return $jobStatus;

    }

    public function serverChange(
        int $serverid,
        int $cpuCores,
        int $mem,
        string $disk,
        int $ipCount
    ) {
        $vmConfig = $this->serverConfig($serverid);

        if ($vmConfig->ipCount > $ipCount) {
            throw new APIException("Downgrading ip count not supported");
        }

        return $this->put("datacenter/server/$serverid/change", [], [
            "cpuCores" => $cpuCores,
            "mem" => $mem * 1024,
            "disk" => $disk . "G",
            "ipCount" => $ipCount
        ]);
    }

    public function serverStatus(int $sid) {
        return $this->get("datacenter/server/$sid/status");
    }
    public function serverConfig(int $sid) {
        return $this->get("datacenter/server/$sid/config");
    }

    public function serverStart(int $sid) {
        return $this->put("datacenter/server/$sid/start");
    }

    public function serverStop(int $sid) {
        return $this->put("datacenter/server/$sid/stop");
    }

    public function serverShutdown(int $sid) {
        return $this->put("datacenter/server/$sid/shutdown");
    }

    public function serverRestart(int $sid) {
        return $this->put("datacenter/server/$sid/reboot");
    }

    public function serverDelete(int $sid, bool $force = false) {
        return $this->post("datacenter/server/$sid/delete", [], ['force' => $force]);
    }

    public function serverBackups(int $sid) {
        return $this->get("datacenter/server/$sid/backups/list");
    }

    public function serverBackupsPending(int $sid) {
        return $this->get("datacenter/server/$sid/backups/status");
    }

    public function serverBackupsCreate(int $sid) {
        return $this->put("datacenter/server/$sid/backups/create");
    }

    public function serverBackupsRestore(int $sid, string $backup) {
        return $this->post("datacenter/server/$sid/backups/restore", [], ['backup' => $backup]);
    }

    public function serverTemplates() {
        return $this->get("datacenter/templates");
    }

    public function serverReinstall($sid, $template) {
        $post = [];

        if ($template) {
            $post['template'] = $template;
        }

        return $this->put("datacenter/server/$sid/reinstall", [], $post);
    }

    public function serverPasswordReset($sid) {
        return $this->put("datacenter/server/$sid/password/reset");
    }

    public function setRdns(int $sid, $ip, $hostname)
    {
        $post['hostname'] = $hostname;
        $post['ip'] = $ip;
        return $this->post('datacenter/server/'.$sid."/rdns", [], $post);
    }

    public function job(int $jobId) {
        return $this->get("job/$jobId");
    }

    public function noVNC($sid) {
        return $this->get("datacenter/server/$sid/console");
    }
}